"use strict";
cc._RF.push(module, 'a6431TpHSRGirMrKTS/aMDZ', 'ResLoader');
// Script/ResLoader.ts

"use strict";
/**
 * 资源加载类
 * 1. 加载完成后自动记录引用关系，根据DependKeys记录反向依赖
 * 2. 支持资源使用，如某打开的UI使用了A资源，其他地方释放资源B，资源B引用了资源A，如果没有其他引用资源A的资源，会触发资源A的释放，
 * 3. 能够安全释放依赖资源（一个资源同时被多个资源引用，只有当其他资源都释放时，该资源才会被释放）
 *
 * 2020-5-25 by Tony.Li
 */
Object.defineProperty(exports, "__esModule", { value: true });
var TAG = '[ResLoader]:';
// 兼容性处理
var isChildClassOf = cc.js["isChildClassOf"];
if (!isChildClassOf) {
    isChildClassOf = cc["isChildClassOf"];
}
var ResLoader = /** @class */ (function () {
    function ResLoader() {
        this._resMap = new Map();
        //console.log(TAG + 'The new object is created!');
    }
    ResLoader.getInstance = function () {
        if (!this._resLoader) {
            this._resLoader = new ResLoader();
        }
        return this._resLoader;
    };
    ResLoader.destroy = function () {
        if (this._resLoader) {
            this._resLoader = null;
        }
    };
    /**
     * loadRes方法的参数预处理
     */
    ResLoader.prototype._makeLoadResArgs = function () {
        if (arguments.length < 1 || typeof arguments[0] != "string") {
            console.error(TAG + ("_makeLoadResArgs error " + arguments));
            return null;
        }
        var ret = { url: arguments[0] };
        for (var i = 1; i < arguments.length; ++i) {
            if (i == 1 && isChildClassOf(arguments[i], cc.Asset)) {
                // 判断是不是第一个参数type
                ret.type = arguments[i];
            }
            else if (i == arguments.length - 1 && typeof arguments[i] == "string") {
                // 判断是不是最后一个参数use
                ret.use = arguments[i];
            }
            else if (typeof arguments[i] == "function") {
                // 其他情况为函数
                if (arguments.length > i + 1 && typeof arguments[i + 1] == "function") {
                    ret.onProgess = arguments[i];
                }
                else {
                    ret.onCompleted = arguments[i];
                }
            }
        }
        return ret;
    };
    /**
     * releaseRes方法的参数预处理
     */
    ResLoader.prototype._makeReleaseResArgs = function () {
        if (arguments.length < 1 || typeof arguments[0] != "string") {
            console.error(TAG + ("_makeReleaseResArgs error " + arguments));
            return null;
        }
        var ret = { url: arguments[0] };
        for (var i = 1; i < arguments.length; ++i) {
            if (typeof arguments[i] == "string") {
                ret.use = arguments[i];
            }
            else {
                ret.type = arguments[i];
            }
        }
        return ret;
    };
    /**
     * 生成一个资源使用Key
     * @param where 在哪里使用，如Scene、UI、Pool
     * @param who 使用者，如Login、UIHelp...
     * @param why 使用原因，自定义...
     */
    ResLoader.makeUseKey = function (where, who, why) {
        if (who === void 0) { who = "none"; }
        if (why === void 0) { why = ""; }
        return "use_" + where + "_by_" + who + "_for_" + why;
    };
    /**
     * 获取资源缓存信息
     * @param key 要获取的资源url
     */
    ResLoader.prototype.getCacheAsset = function (key) {
        if (!this._resMap.has(key)) {
            return null;
        }
        console.log("key==============>>>>", key);
        return this._resMap.get(key);
    };
    ResLoader.prototype.loadRes = function () {
        var _this = this;
        var resArgs = this._makeLoadResArgs.apply(this, arguments);
        console.log(TAG + "loadRes2222222|" + resArgs.url);
        var finishCallback = function (error, asset) {
            if (error) {
                console.log(error.message);
            }
            else {
                console.log(TAG + ">releaseRes11111: res refCount:" + asset.refCount);
                asset.addRef();
                _this._resMap.set(resArgs.url, asset);
            }
            // 执行完成回调
            if (resArgs.onCompleted) {
                resArgs.onCompleted(error, asset);
            }
            console.timeEnd(TAG + "loadRes1111|" + resArgs.url);
        };
        // 预判是否资源已加载
        var res = cc.resources.get(resArgs.url, resArgs.type);
        console.log("res=============>>>>>", res);
        if (res) {
            console.log("=========================have================");
            finishCallback(null, res);
        }
        else {
            cc.resources.load(resArgs.url, resArgs.type, resArgs.onProgess, finishCallback);
        }
    };
    ResLoader.prototype.releaseRes = function () {
        var resArgs = this._makeReleaseResArgs.apply(this, arguments);
        // cc.resources.release(resArgs.url);
        console.log(TAG + ">releaseRes: " + resArgs.url);
        var cacheAsset = this.getCacheAsset(resArgs.url);
        if (cacheAsset) {
            // cc.assetManager.releaseAsset(cacheAsset);
            cacheAsset.decRef();
            console.log(TAG + ">releaseRes: res refCount:" + cacheAsset.refCount);
            if (cacheAsset.refCount == 0) {
                cacheAsset = null;
                this._resMap.delete(resArgs.url);
            }
            cacheAsset = null;
        }
    };
    ResLoader._resLoader = null;
    return ResLoader;
}());
exports.default = ResLoader;

cc._RF.pop();